@@ -351,7 +351,7 @@ class Debugger:
         code = message['arguments']['code']
         file_name = get_file_name(code)
 
-        with open(file_name, 'w') as f:
+        with open(file_name, 'w', encoding='utf-8') as f:
             f.write(code)
 
         reply = {
@@ -378,7 +378,7 @@ class Debugger:
         }
         source_path = message["arguments"]["source"]["path"]
         if os.path.isfile(source_path):
-            with open(source_path) as f:
+            with open(source_path, encoding='utf-8') as f:
                 reply['success'] = True
                 reply['body'] = {
                     'content': f.read()
@@ -462,6 +462,8 @@ class Debugger:
             'port': port
         }
         message['arguments']['logToFile'] = True
+        # Set debugOptions for breakpoints in python standard library source.
+        message['arguments']['debugOptions'] = [ 'DebugStdLib' ]
         return await self._forward_message(message)
 
     async def configurationDone(self, message):
@@ -494,7 +496,8 @@ class Debugger:
                 'tmpFileSuffix': '.py',
                 'breakpoints': breakpoint_list,
                 'stoppedThreads': self.stopped_threads,
-                'richRendering': True
+                'richRendering': True,
+                'exceptionPaths': ['Python Exceptions']
             }
         }
         return reply
